﻿using DressAtelierContracts.BindingModels;
using DressAtelierContracts.SearchModels;
using DressAtelierContracts.StorageContracts;
using DressAtelierContracts.ViewModels;
using DressAtelierDatabaseImplement.Models;
using DressAtelierDatabaseImplementation;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DressAtelierDatabaseImplement.Implements
{
    public class MessageInfoStorage : IMessageInfoStorage
    {
        public MessageInfoViewModel? GetElement(MessageInfoSearchModel model)
        {
            if(string.IsNullOrEmpty(model.ID))
            {
                return null;
            }
            using var context = new DressAtelierDatabase();
            return context.Messages.Include(x => x.Client).FirstOrDefault(x => x.ID.Equals(model.ID))?.GetViewModel;
        }

        public List<MessageInfoViewModel> GetFilteredList(MessageInfoSearchModel model)
        {
            if (!model.ClientID.HasValue)
            {
                return new();
            }
            using var context = new DressAtelierDatabase();
            return context.Messages.Include(x => x.Client).Where(x => x.ClientID == model.ClientID).Select(x => x.GetViewModel).ToList() ;
        }

        public List<MessageInfoViewModel> GetFullList()
        {
            using var context = new DressAtelierDatabase();
            return context.Messages.Include(x => x.Client).Select(x => x.GetViewModel).ToList();
        }

        public MessageInfoViewModel? Insert(MessageInfoBindingModel model)
        {

            using var context = new DressAtelierDatabase();

            model.ClientID = context.Clients.FirstOrDefault(x => x.Email.Equals(model.SenderName))?.ID;

            var message = MessageInfo.Create(model);
            if (message == null)
            {
                return null;
            }

            if (GetElement(new MessageInfoSearchModel { ID = model.ID }) != null)
            {
                return null;
            }

            context.Messages.Add(message);
            context.SaveChanges();
            return message.GetViewModel;
        }
    }
}
