using DressAtelierBusinessLogic.BusinessLogic;
using DressAtelierBusinessLogic.MailEmployee;
using DressAtelierContracts.BindingModels;
using DressAtelierContracts.BusinessLogicContracts;
using DressAtelierContracts.StorageContracts;
using DressAtelierDatabaseImplement.Implements;
using DressAtelierDatabaseImplementation.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.OpenApi.Models;

var builder = WebApplication.CreateBuilder(args);

builder.Logging.SetMinimumLevel(LogLevel.Trace);
builder.Logging.AddLog4Net("log4net.config");

// Add services to the container.

builder.Services.AddTransient<IClientStorage, ClientStorage>();
builder.Services.AddTransient<IOrderStorage, OrderStorage>();
builder.Services.AddTransient<IDressStorage, DressStorage>();
builder.Services.AddTransient<IEmployeeStorage,EmployeeStorage>();
builder.Services.AddTransient<IMessageInfoStorage, MessageInfoStorage>();

builder.Services.AddTransient<IOrderLogic, OrderLogic>();
builder.Services.AddTransient<IClientLogic, ClientLogic>();
builder.Services.AddTransient<IDressLogic, DressLogic>();
builder.Services.AddTransient<IEmployeeLogic, EmployeeLogic>();
builder.Services.AddTransient<IMessageInfoLogic,MessageInfoLogic>();

builder.Services.AddSingleton<AbstractMailEmployee,MailKitEmployee>();

builder.Services.AddControllers();
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
    c.SwaggerDoc("v1", new OpenApiInfo { Title = "DressAtelierRestApi", Version = "v1" });
});

var app = builder.Build();
var mailSender = app.Services.GetService<AbstractMailEmployee>();
mailSender?.MailConfig(new MailConfigBindingModel 
{ 
    Login = builder.Configuration?.GetSection("MailLogin")?.Value?.ToString() ?? string.Empty, 
    Password = builder.Configuration?.GetSection("MailPassword")?.Value?.ToString() ?? string.Empty, 
    SmtpClientHost = builder.Configuration?.GetSection("SmtpClientHost")?.Value?.ToString() ?? string.Empty, 
    SmtpClientPort = Convert.ToInt32(builder.Configuration?.GetSection("SmtpClientPort")?.Value?.ToString()), 
    PopHost = builder.Configuration?.GetSection("PopHost")?.Value?.ToString() ?? string.Empty, 
    PopPort = Convert.ToInt32(builder.Configuration?.GetSection("PopPort")?.Value?.ToString()) 
});

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();
    app.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json", "DressAtelierRestApi v1"));
}

app.UseHttpsRedirection();

app.UseAuthorization();

app.MapControllers();

app.Run();
