using DressAtelierBusinessLogic.BusinessLogic;
using DressAtelierBusinessLogic.MailEmployee;
using DressAtelierBusinessLogic.OfficePackage;
using DressAtelierBusinessLogic.OfficePackage.Implements;
using DressAtelierContracts.BindingModels;
using DressAtelierContracts.BusinessLogicContracts;
using DressAtelierContracts.StorageContracts;
using DressAtelierDatabaseImplement.Implements;
using DressAtelierDatabaseImplementation.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;

namespace SewingDresses
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;

        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();

            try
            {
                var mailSender = _serviceProvider.GetService<AbstractMailEmployee>(); 
                mailSender?.MailConfig(new MailConfigBindingModel 
                {
                    Login = System.Configuration.ConfigurationManager.AppSettings["MailLogin"] ?? string.Empty, 
                    Password = System.Configuration.ConfigurationManager.AppSettings["MailPassword"] ?? string.Empty, 
                    SmtpClientHost = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"] ?? string.Empty, 
                    SmtpClientPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"]), 
                    PopHost = System.Configuration.ConfigurationManager.AppSettings["PopHost"] ?? string.Empty, 
                    PopPort = Convert.ToInt32(System.Configuration.ConfigurationManager.AppSettings["PopPort"]) 
                });      
                //  
                var timer = new System.Threading.Timer(new TimerCallback(MailCheck!), null, 0, 100000);    
            }    
            catch(Exception ex)             
            {     
                var logger = _serviceProvider.GetService<ILogger>();     
                logger?.LogError(ex, "Working with email error");    
            } 

            Application.Run(_serviceProvider.GetRequiredService<FormMain>());
        }

        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IMaterialStorage, MaterialStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IDressStorage, DressStorage>();
            services.AddTransient<IClientStorage, ClientStorage>();
            services.AddTransient<IEmployeeStorage, EmployeeStorage>();
            services.AddTransient<IMessageInfoStorage,MessageInfoStorage>();

            services.AddTransient<IMaterialLogic, MaterialLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IDressLogic, DressLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IClientLogic, ClientLogic>();
            services.AddTransient<IEmployeeLogic, EmployeeLogic>();
            services.AddTransient<IMessageInfoLogic, MessageInfoLogic>();

            services.AddTransient<IWorkImitation, WorkImitation>();
            services.AddSingleton<AbstractMailEmployee, MailKitEmployee>();

            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            services.AddTransient<FormMain>();
            services.AddTransient<FormMaterial>();
            services.AddTransient<FormMaterials>();
            services.AddTransient<FormOrderCreation>();
            services.AddTransient<FormDress>();
            services.AddTransient<FormDressMaterial>();
            services.AddTransient<FormDresses>();
            services.AddTransient<FormReportDressMaterials>();
            services.AddTransient<FormReportOrders>();
            services.AddTransient<FormClients>();
            services.AddTransient<FormEmployees>();
            services.AddTransient<FormEmployee>();
            services.AddTransient<FormEmails>();
        }

        private static void MailCheck(object obj) => ServiceProvider?.GetService<AbstractMailEmployee>()?.MailCheck();

    }
}